// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace Media { namespace MediaProperties { 
  v8::Handle<v8::Value> WrapMediaRatio(::Windows::Media::MediaProperties::MediaRatio^ wintRtInstance);
  ::Windows::Media::MediaProperties::MediaRatio^ UnwrapMediaRatio(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMediaPropertySet(::Windows::Media::MediaProperties::MediaPropertySet^ wintRtInstance);
  ::Windows::Media::MediaProperties::MediaPropertySet^ UnwrapMediaPropertySet(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIMediaEncodingProperties(::Windows::Media::MediaProperties::IMediaEncodingProperties^ wintRtInstance);
  ::Windows::Media::MediaProperties::IMediaEncodingProperties^ UnwrapIMediaEncodingProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapAudioEncodingProperties(::Windows::Media::MediaProperties::AudioEncodingProperties^ wintRtInstance);
  ::Windows::Media::MediaProperties::AudioEncodingProperties^ UnwrapAudioEncodingProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMediaEncodingSubtypes(::Windows::Media::MediaProperties::MediaEncodingSubtypes^ wintRtInstance);
  ::Windows::Media::MediaProperties::MediaEncodingSubtypes^ UnwrapMediaEncodingSubtypes(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapH264ProfileIds(::Windows::Media::MediaProperties::H264ProfileIds^ wintRtInstance);
  ::Windows::Media::MediaProperties::H264ProfileIds^ UnwrapH264ProfileIds(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMpeg2ProfileIds(::Windows::Media::MediaProperties::Mpeg2ProfileIds^ wintRtInstance);
  ::Windows::Media::MediaProperties::Mpeg2ProfileIds^ UnwrapMpeg2ProfileIds(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapVideoEncodingProperties(::Windows::Media::MediaProperties::VideoEncodingProperties^ wintRtInstance);
  ::Windows::Media::MediaProperties::VideoEncodingProperties^ UnwrapVideoEncodingProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageEncodingProperties(::Windows::Media::MediaProperties::ImageEncodingProperties^ wintRtInstance);
  ::Windows::Media::MediaProperties::ImageEncodingProperties^ UnwrapImageEncodingProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapContainerEncodingProperties(::Windows::Media::MediaProperties::ContainerEncodingProperties^ wintRtInstance);
  ::Windows::Media::MediaProperties::ContainerEncodingProperties^ UnwrapContainerEncodingProperties(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapMediaEncodingProfile(::Windows::Media::MediaProperties::MediaEncodingProfile^ wintRtInstance);
  ::Windows::Media::MediaProperties::MediaEncodingProfile^ UnwrapMediaEncodingProfile(Handle<Value> value);
  


  static v8::Handle<v8::Value> InitMediaPixelFormatEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("MediaPixelFormat"), enumObject);

    enumObject->Set(String::NewSymbol("nv12"), Integer::New(0));
    enumObject->Set(String::NewSymbol("bgra8"), Integer::New(1));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitMediaThumbnailFormatEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("MediaThumbnailFormat"), enumObject);

    enumObject->Set(String::NewSymbol("bmp"), Integer::New(0));
    enumObject->Set(String::NewSymbol("bgra8"), Integer::New(1));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitAudioEncodingQualityEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("AudioEncodingQuality"), enumObject);

    enumObject->Set(String::NewSymbol("auto"), Integer::New(0));
    enumObject->Set(String::NewSymbol("high"), Integer::New(1));
    enumObject->Set(String::NewSymbol("medium"), Integer::New(2));
    enumObject->Set(String::NewSymbol("low"), Integer::New(3));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitVideoEncodingQualityEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("VideoEncodingQuality"), enumObject);

    enumObject->Set(String::NewSymbol("auto"), Integer::New(0));
    enumObject->Set(String::NewSymbol("hD1080p"), Integer::New(1));
    enumObject->Set(String::NewSymbol("hD720p"), Integer::New(2));
    enumObject->Set(String::NewSymbol("wvga"), Integer::New(3));
    enumObject->Set(String::NewSymbol("ntsc"), Integer::New(4));
    enumObject->Set(String::NewSymbol("pal"), Integer::New(5));
    enumObject->Set(String::NewSymbol("vga"), Integer::New(6));
    enumObject->Set(String::NewSymbol("qvga"), Integer::New(7));

    return scope.Close(Undefined());
  }



  
  class MediaRatio : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaRatio"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("numerator"), NumeratorGetter, NumeratorSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("denominator"), DenominatorGetter, DenominatorSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MediaRatio"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaRatio(::Windows::Media::MediaProperties::MediaRatio^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::MediaRatio^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaRatio^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::MediaRatio^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaRatio *wrapperInstance = new MediaRatio(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> NumeratorGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaRatio^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MediaRatio *wrapper = MediaRatio::Unwrap<MediaRatio>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Numerator;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void NumeratorSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaRatio^>(info.This()))
      {
        return;
      }

      MediaRatio *wrapper = MediaRatio::Unwrap<MediaRatio>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Numerator = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DenominatorGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaRatio^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MediaRatio *wrapper = MediaRatio::Unwrap<MediaRatio>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Denominator;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void DenominatorSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaRatio^>(info.This()))
      {
        return;
      }

      MediaRatio *wrapper = MediaRatio::Unwrap<MediaRatio>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Denominator = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::MediaRatio^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaRatio(::Windows::Media::MediaProperties::MediaRatio^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::MediaRatio^ UnwrapMediaRatio(Handle<Value> value);
    friend bool IsMediaRatioWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaRatio::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaRatio(::Windows::Media::MediaProperties::MediaRatio^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaRatio::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::MediaRatio^ UnwrapMediaRatio(Handle<Value> value)
  {
     return MediaRatio::Unwrap<MediaRatio>(value.As<Object>())->_instance;
  }

  void InitMediaRatio(Handle<Object> exports)
  {
    MediaRatio::Init(exports);
  }

  class MediaPropertySet : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaPropertySet"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("lookup"), FunctionTemplate::New(Lookup)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("hasKey"), FunctionTemplate::New(HasKey)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getView"), FunctionTemplate::New(GetView)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("insert"), FunctionTemplate::New(Insert)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("remove"), FunctionTemplate::New(Remove)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("clear"), FunctionTemplate::New(Clear)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("first"), FunctionTemplate::New(First)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("MediaPropertySet"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaPropertySet(::Windows::Media::MediaProperties::MediaPropertySet^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::MediaPropertySet^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::MediaPropertySet^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaProperties::MediaPropertySet();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaPropertySet *wrapperInstance = new MediaPropertySet(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> Lookup(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsGuid(args[0]))
      {
        try
        {
          ::Platform::Guid arg0 = NodeRT::Utils::GuidFromJs(args[0]);
          
          ::Platform::Object^ result;
          result = wrapper->_instance->Lookup(arg0);
          return scope.Close(CreateOpaqueWrapper(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> HasKey(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsGuid(args[0]))
      {
        try
        {
          ::Platform::Guid arg0 = NodeRT::Utils::GuidFromJs(args[0]);
          
          bool result;
          result = wrapper->_instance->HasKey(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetView(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Foundation::Collections::IMapView<::Platform::Guid, ::Platform::Object^>^ result;
          result = wrapper->_instance->GetView();
          return scope.Close(NodeRT::Collections::MapViewWrapper<::Platform::Guid,::Platform::Object^>::CreateMapViewWrapper(result, 
            [](::Platform::Guid val) -> Handle<Value> {
              return NodeRT::Utils::GuidToJs(val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsGuid(value);
            },
            [](Handle<Value> value) -> ::Platform::Guid {
              return NodeRT::Utils::GuidFromJs(value);
            },
            [](::Platform::Object^ val) -> Handle<Value> {
              return CreateOpaqueWrapper(val);
            }
          ));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Insert(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 2
        && NodeRT::Utils::IsGuid(args[0])
        && NodeRT::Utils::IsWinRtWrapperOf<::Platform::Object^>(args[1]))
      {
        try
        {
          ::Platform::Guid arg0 = NodeRT::Utils::GuidFromJs(args[0]);
          ::Platform::Object^ arg1 = dynamic_cast<::Platform::Object^>(NodeRT::Utils::GetObjectInstance(args[1]));
          
          bool result;
          result = wrapper->_instance->Insert(arg0, arg1);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Remove(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsGuid(args[0]))
      {
        try
        {
          ::Platform::Guid arg0 = NodeRT::Utils::GuidFromJs(args[0]);
          
          wrapper->_instance->Remove(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Clear(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->Clear();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> First(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaPropertySet^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      MediaPropertySet *wrapper = MediaPropertySet::Unwrap<MediaPropertySet>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Foundation::Collections::IIterator<::Windows::Foundation::Collections::IKeyValuePair<::Platform::Guid, ::Platform::Object^>^>^ result;
          result = wrapper->_instance->First();
          return scope.Close(NodeRT::Collections::IteratorWrapper<::Windows::Foundation::Collections::IKeyValuePair<::Platform::Guid, ::Platform::Object^>^>::CreateIteratorWrapper(result, 
            [](::Windows::Foundation::Collections::IKeyValuePair<::Platform::Guid, ::Platform::Object^>^ val) -> Handle<Value> {
              return NodeRT::Collections::KeyValuePairWrapper<::Platform::Guid,::Platform::Object^>::CreateKeyValuePairWrapper(val, 
            [](::Platform::Guid val) -> Handle<Value> {
              return NodeRT::Utils::GuidToJs(val);
            },
            [](::Platform::Object^ val) -> Handle<Value> {
              return CreateOpaqueWrapper(val);
            }
          );
            }
          ));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::Media::MediaProperties::MediaPropertySet^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaPropertySet(::Windows::Media::MediaProperties::MediaPropertySet^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::MediaPropertySet^ UnwrapMediaPropertySet(Handle<Value> value);
    friend bool IsMediaPropertySetWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaPropertySet::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaPropertySet(::Windows::Media::MediaProperties::MediaPropertySet^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaPropertySet::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::MediaPropertySet^ UnwrapMediaPropertySet(Handle<Value> value)
  {
     return MediaPropertySet::Unwrap<MediaPropertySet>(value.As<Object>())->_instance;
  }

  void InitMediaPropertySet(Handle<Object> exports)
  {
    MediaPropertySet::Init(exports);
  }

  class IMediaEncodingProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IMediaEncodingProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("properties"), PropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtype"), SubtypeGetter, SubtypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("type"), TypeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IMediaEncodingProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IMediaEncodingProperties(::Windows::Media::MediaProperties::IMediaEncodingProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::IMediaEncodingProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::IMediaEncodingProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::IMediaEncodingProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IMediaEncodingProperties *wrapperInstance = new IMediaEncodingProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> PropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::IMediaEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaEncodingProperties *wrapper = IMediaEncodingProperties::Unwrap<IMediaEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaPropertySet^ result = wrapper->_instance->Properties;
        return scope.Close(WrapMediaPropertySet(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SubtypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::IMediaEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaEncodingProperties *wrapper = IMediaEncodingProperties::Unwrap<IMediaEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtype;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::IMediaEncodingProperties^>(info.This()))
      {
        return;
      }

      IMediaEncodingProperties *wrapper = IMediaEncodingProperties::Unwrap<IMediaEncodingProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtype = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> TypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::IMediaEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IMediaEncodingProperties *wrapper = IMediaEncodingProperties::Unwrap<IMediaEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Type;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::IMediaEncodingProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIMediaEncodingProperties(::Windows::Media::MediaProperties::IMediaEncodingProperties^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::IMediaEncodingProperties^ UnwrapIMediaEncodingProperties(Handle<Value> value);
    friend bool IsIMediaEncodingPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IMediaEncodingProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIMediaEncodingProperties(::Windows::Media::MediaProperties::IMediaEncodingProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IMediaEncodingProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::IMediaEncodingProperties^ UnwrapIMediaEncodingProperties(Handle<Value> value)
  {
     return IMediaEncodingProperties::Unwrap<IMediaEncodingProperties>(value.As<Object>())->_instance;
  }

  void InitIMediaEncodingProperties(Handle<Object> exports)
  {
    IMediaEncodingProperties::Init(exports);
  }

  class AudioEncodingProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("AudioEncodingProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setFormatUserData"), FunctionTemplate::New(SetFormatUserData)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getFormatUserData"), FunctionTemplate::New(GetFormatUserData)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtype"), SubtypeGetter, SubtypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("properties"), PropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("type"), TypeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("sampleRate"), SampleRateGetter, SampleRateSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("channelCount"), ChannelCountGetter, ChannelCountSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bitsPerSample"), BitsPerSampleGetter, BitsPerSampleSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bitrate"), BitrateGetter, BitrateSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createAac"), FunctionTemplate::New(CreateAac)->GetFunction());
      constructor->Set(String::NewSymbol("createAacAdts"), FunctionTemplate::New(CreateAacAdts)->GetFunction());
      constructor->Set(String::NewSymbol("createMp3"), FunctionTemplate::New(CreateMp3)->GetFunction());
      constructor->Set(String::NewSymbol("createPcm"), FunctionTemplate::New(CreatePcm)->GetFunction());
      constructor->Set(String::NewSymbol("createWma"), FunctionTemplate::New(CreateWma)->GetFunction());

      exports->Set(String::NewSymbol("AudioEncodingProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    AudioEncodingProperties(::Windows::Media::MediaProperties::AudioEncodingProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::AudioEncodingProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::AudioEncodingProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaProperties::AudioEncodingProperties();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      AudioEncodingProperties *wrapperInstance = new AudioEncodingProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> SetFormatUserData(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(args.This());

      if (args.Length() == 1
        && (NodeRT::Utils::IsWinRtWrapperOf<::Platform::Array<unsigned char>^>(args[0]) || args[0]->IsArray()))
      {
        try
        {
          ::Platform::Array<unsigned char>^ arg0 = 
            [] (v8::Handle<v8::Value> value) -> ::Platform::Array<unsigned char>^
            {
              if (value->IsArray())
              {
                return NodeRT::Collections::JsArrayToWinrtArray<unsigned char>(value.As<Array>(), 
                 [](Handle<Value> value) -> bool {
                   return value->IsInt32();
                 },
                 [](Handle<Value> value) -> unsigned char {
                   return static_cast<unsigned char>(value->Int32Value());
                 }
                );
              }
              else
              {
                return dynamic_cast<::Platform::Array<unsigned char>^>(NodeRT::Utils::GetObjectInstance(value));
              }
            } (args[0]);
          
          wrapper->_instance->SetFormatUserData(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetFormatUserData(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Platform::Array<unsigned char>^ arg0;
          
          wrapper->_instance->GetFormatUserData(&arg0);
          Handle<Object> resObj = Object::New();
          resObj->Set(String::NewSymbol("value"), NodeRT::Collections::ArrayWrapper<unsigned char>::CreateArrayWrapper(arg0, 
            [](unsigned char val) -> Handle<Value> {
              return Integer::New(val);
            },
            [](Handle<Value> value) -> bool {
              return value->IsInt32();
            },
            [](Handle<Value> value) -> unsigned char {
              return static_cast<unsigned char>(value->Int32Value());
            }
          ));
          return scope.Close(resObj);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }


    static Handle<Value> CreateAac(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsUint32()
        && args[1]->IsUint32()
        && args[2]->IsUint32())
      {
        try
        {
          unsigned int arg0 = static_cast<unsigned int>(args[0]->IntegerValue());
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          unsigned int arg2 = static_cast<unsigned int>(args[2]->IntegerValue());
          
          ::Windows::Media::MediaProperties::AudioEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::AudioEncodingProperties::CreateAac(arg0, arg1, arg2);
          return scope.Close(WrapAudioEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateAacAdts(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsUint32()
        && args[1]->IsUint32()
        && args[2]->IsUint32())
      {
        try
        {
          unsigned int arg0 = static_cast<unsigned int>(args[0]->IntegerValue());
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          unsigned int arg2 = static_cast<unsigned int>(args[2]->IntegerValue());
          
          ::Windows::Media::MediaProperties::AudioEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::AudioEncodingProperties::CreateAacAdts(arg0, arg1, arg2);
          return scope.Close(WrapAudioEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateMp3(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsUint32()
        && args[1]->IsUint32()
        && args[2]->IsUint32())
      {
        try
        {
          unsigned int arg0 = static_cast<unsigned int>(args[0]->IntegerValue());
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          unsigned int arg2 = static_cast<unsigned int>(args[2]->IntegerValue());
          
          ::Windows::Media::MediaProperties::AudioEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::AudioEncodingProperties::CreateMp3(arg0, arg1, arg2);
          return scope.Close(WrapAudioEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreatePcm(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsUint32()
        && args[1]->IsUint32()
        && args[2]->IsUint32())
      {
        try
        {
          unsigned int arg0 = static_cast<unsigned int>(args[0]->IntegerValue());
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          unsigned int arg2 = static_cast<unsigned int>(args[2]->IntegerValue());
          
          ::Windows::Media::MediaProperties::AudioEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::AudioEncodingProperties::CreatePcm(arg0, arg1, arg2);
          return scope.Close(WrapAudioEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateWma(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsUint32()
        && args[1]->IsUint32()
        && args[2]->IsUint32())
      {
        try
        {
          unsigned int arg0 = static_cast<unsigned int>(args[0]->IntegerValue());
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          unsigned int arg2 = static_cast<unsigned int>(args[2]->IntegerValue());
          
          ::Windows::Media::MediaProperties::AudioEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::AudioEncodingProperties::CreateWma(arg0, arg1, arg2);
          return scope.Close(WrapAudioEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> SubtypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtype;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return;
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtype = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> PropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaPropertySet^ result = wrapper->_instance->Properties;
        return scope.Close(WrapMediaPropertySet(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Type;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SampleRateGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->SampleRate;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SampleRateSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return;
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->SampleRate = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ChannelCountGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->ChannelCount;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ChannelCountSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return;
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->ChannelCount = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BitsPerSampleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->BitsPerSample;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void BitsPerSampleSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return;
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->BitsPerSample = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BitrateGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Bitrate;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void BitrateSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(info.This()))
      {
        return;
      }

      AudioEncodingProperties *wrapper = AudioEncodingProperties::Unwrap<AudioEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Bitrate = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::AudioEncodingProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapAudioEncodingProperties(::Windows::Media::MediaProperties::AudioEncodingProperties^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::AudioEncodingProperties^ UnwrapAudioEncodingProperties(Handle<Value> value);
    friend bool IsAudioEncodingPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> AudioEncodingProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapAudioEncodingProperties(::Windows::Media::MediaProperties::AudioEncodingProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(AudioEncodingProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::AudioEncodingProperties^ UnwrapAudioEncodingProperties(Handle<Value> value)
  {
     return AudioEncodingProperties::Unwrap<AudioEncodingProperties>(value.As<Object>())->_instance;
  }

  void InitAudioEncodingProperties(Handle<Object> exports)
  {
    AudioEncodingProperties::Init(exports);
  }

  class MediaEncodingSubtypes : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaEncodingSubtypes"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->SetAccessor(String::NewSymbol("aac"), AacGetter);
      constructor->SetAccessor(String::NewSymbol("aacAdts"), AacAdtsGetter);
      constructor->SetAccessor(String::NewSymbol("ac3"), Ac3Getter);
      constructor->SetAccessor(String::NewSymbol("amrNb"), AmrNbGetter);
      constructor->SetAccessor(String::NewSymbol("amrWb"), AmrWbGetter);
      constructor->SetAccessor(String::NewSymbol("argb32"), Argb32Getter);
      constructor->SetAccessor(String::NewSymbol("asf"), AsfGetter);
      constructor->SetAccessor(String::NewSymbol("avi"), AviGetter);
      constructor->SetAccessor(String::NewSymbol("bgra8"), Bgra8Getter);
      constructor->SetAccessor(String::NewSymbol("bmp"), BmpGetter);
      constructor->SetAccessor(String::NewSymbol("float"), FloatGetter);
      constructor->SetAccessor(String::NewSymbol("gif"), GifGetter);
      constructor->SetAccessor(String::NewSymbol("h263"), H263Getter);
      constructor->SetAccessor(String::NewSymbol("h264"), H264Getter);
      constructor->SetAccessor(String::NewSymbol("h264Es"), H264EsGetter);
      constructor->SetAccessor(String::NewSymbol("hevc"), HevcGetter);
      constructor->SetAccessor(String::NewSymbol("hevcEs"), HevcEsGetter);
      constructor->SetAccessor(String::NewSymbol("iyuv"), IyuvGetter);
      constructor->SetAccessor(String::NewSymbol("jpeg"), JpegGetter);
      constructor->SetAccessor(String::NewSymbol("jpegXr"), JpegXrGetter);
      constructor->SetAccessor(String::NewSymbol("mjpg"), MjpgGetter);
      constructor->SetAccessor(String::NewSymbol("mp3"), Mp3Getter);
      constructor->SetAccessor(String::NewSymbol("mpeg"), MpegGetter);
      constructor->SetAccessor(String::NewSymbol("mpeg1"), Mpeg1Getter);
      constructor->SetAccessor(String::NewSymbol("mpeg2"), Mpeg2Getter);
      constructor->SetAccessor(String::NewSymbol("mpeg4"), Mpeg4Getter);
      constructor->SetAccessor(String::NewSymbol("nv12"), Nv12Getter);
      constructor->SetAccessor(String::NewSymbol("pcm"), PcmGetter);
      constructor->SetAccessor(String::NewSymbol("png"), PngGetter);
      constructor->SetAccessor(String::NewSymbol("rgb32"), Rgb32Getter);
      constructor->SetAccessor(String::NewSymbol("tiff"), TiffGetter);
      constructor->SetAccessor(String::NewSymbol("wave"), WaveGetter);
      constructor->SetAccessor(String::NewSymbol("wma8"), Wma8Getter);
      constructor->SetAccessor(String::NewSymbol("wma9"), Wma9Getter);
      constructor->SetAccessor(String::NewSymbol("wmv3"), Wmv3Getter);
      constructor->SetAccessor(String::NewSymbol("wvc1"), Wvc1Getter);
      constructor->SetAccessor(String::NewSymbol("yuy2"), Yuy2Getter);
      constructor->SetAccessor(String::NewSymbol("yv12"), Yv12Getter);
      constructor->SetAccessor(String::NewSymbol("rgb24"), Rgb24Getter);
      constructor->SetAccessor(String::NewSymbol("eac3"), Eac3Getter);

      exports->Set(String::NewSymbol("MediaEncodingSubtypes"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaEncodingSubtypes(::Windows::Media::MediaProperties::MediaEncodingSubtypes^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::MediaEncodingSubtypes^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingSubtypes^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::MediaEncodingSubtypes^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaEncodingSubtypes *wrapperInstance = new MediaEncodingSubtypes(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  




    static Handle<Value> AacGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Aac;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> AacAdtsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::AacAdts;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Ac3Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Ac3;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> AmrNbGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::AmrNb;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> AmrWbGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::AmrWb;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Argb32Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Argb32;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> AsfGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Asf;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> AviGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Avi;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Bgra8Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Bgra8;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> BmpGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Bmp;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> FloatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Float;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> GifGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Gif;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> H263Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::H263;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> H264Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::H264;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> H264EsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::H264Es;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> HevcGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Hevc;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> HevcEsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::HevcEs;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> IyuvGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Iyuv;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> JpegGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Jpeg;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> JpegXrGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::JpegXr;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MjpgGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Mjpg;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Mp3Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Mp3;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MpegGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Mpeg;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Mpeg1Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Mpeg1;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Mpeg2Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Mpeg2;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Mpeg4Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Mpeg4;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Nv12Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Nv12;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> PcmGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Pcm;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> PngGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Png;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Rgb32Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Rgb32;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TiffGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Tiff;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> WaveGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Wave;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Wma8Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Wma8;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Wma9Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Wma9;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Wmv3Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Wmv3;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Wvc1Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Wvc1;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Yuy2Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Yuy2;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Yv12Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Yv12;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Rgb24Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Rgb24;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> Eac3Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        Platform::String^ result = ::Windows::Media::MediaProperties::MediaEncodingSubtypes::Eac3;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    

  private:
    ::Windows::Media::MediaProperties::MediaEncodingSubtypes^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaEncodingSubtypes(::Windows::Media::MediaProperties::MediaEncodingSubtypes^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::MediaEncodingSubtypes^ UnwrapMediaEncodingSubtypes(Handle<Value> value);
    friend bool IsMediaEncodingSubtypesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaEncodingSubtypes::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaEncodingSubtypes(::Windows::Media::MediaProperties::MediaEncodingSubtypes^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaEncodingSubtypes::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::MediaEncodingSubtypes^ UnwrapMediaEncodingSubtypes(Handle<Value> value)
  {
     return MediaEncodingSubtypes::Unwrap<MediaEncodingSubtypes>(value.As<Object>())->_instance;
  }

  void InitMediaEncodingSubtypes(Handle<Object> exports)
  {
    MediaEncodingSubtypes::Init(exports);
  }

  class H264ProfileIds : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("H264ProfileIds"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->SetAccessor(String::NewSymbol("baseline"), BaselineGetter);
      constructor->SetAccessor(String::NewSymbol("constrainedBaseline"), ConstrainedBaselineGetter);
      constructor->SetAccessor(String::NewSymbol("extended"), ExtendedGetter);
      constructor->SetAccessor(String::NewSymbol("high"), HighGetter);
      constructor->SetAccessor(String::NewSymbol("high10"), High10Getter);
      constructor->SetAccessor(String::NewSymbol("high422"), High422Getter);
      constructor->SetAccessor(String::NewSymbol("high444"), High444Getter);
      constructor->SetAccessor(String::NewSymbol("main"), MainGetter);
      constructor->SetAccessor(String::NewSymbol("multiviewHigh"), MultiviewHighGetter);
      constructor->SetAccessor(String::NewSymbol("stereoHigh"), StereoHighGetter);

      exports->Set(String::NewSymbol("H264ProfileIds"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    H264ProfileIds(::Windows::Media::MediaProperties::H264ProfileIds^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::H264ProfileIds^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::H264ProfileIds^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::H264ProfileIds^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      H264ProfileIds *wrapperInstance = new H264ProfileIds(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  




    static Handle<Value> BaselineGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::Baseline;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ConstrainedBaselineGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::ConstrainedBaseline;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ExtendedGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::Extended;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> HighGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::High;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> High10Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::High10;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> High422Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::High422;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> High444Getter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::High444;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MainGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::Main;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MultiviewHighGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::MultiviewHigh;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> StereoHighGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::H264ProfileIds::StereoHigh;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    

  private:
    ::Windows::Media::MediaProperties::H264ProfileIds^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapH264ProfileIds(::Windows::Media::MediaProperties::H264ProfileIds^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::H264ProfileIds^ UnwrapH264ProfileIds(Handle<Value> value);
    friend bool IsH264ProfileIdsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> H264ProfileIds::s_constructorTemplate;

  v8::Handle<v8::Value> WrapH264ProfileIds(::Windows::Media::MediaProperties::H264ProfileIds^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(H264ProfileIds::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::H264ProfileIds^ UnwrapH264ProfileIds(Handle<Value> value)
  {
     return H264ProfileIds::Unwrap<H264ProfileIds>(value.As<Object>())->_instance;
  }

  void InitH264ProfileIds(Handle<Object> exports)
  {
    H264ProfileIds::Init(exports);
  }

  class Mpeg2ProfileIds : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("Mpeg2ProfileIds"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->SetAccessor(String::NewSymbol("high"), HighGetter);
      constructor->SetAccessor(String::NewSymbol("main"), MainGetter);
      constructor->SetAccessor(String::NewSymbol("signalNoiseRatioScalable"), SignalNoiseRatioScalableGetter);
      constructor->SetAccessor(String::NewSymbol("simple"), SimpleGetter);
      constructor->SetAccessor(String::NewSymbol("spatiallyScalable"), SpatiallyScalableGetter);

      exports->Set(String::NewSymbol("Mpeg2ProfileIds"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    Mpeg2ProfileIds(::Windows::Media::MediaProperties::Mpeg2ProfileIds^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::Mpeg2ProfileIds^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::Mpeg2ProfileIds^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::Mpeg2ProfileIds^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      Mpeg2ProfileIds *wrapperInstance = new Mpeg2ProfileIds(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  




    static Handle<Value> HighGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::Mpeg2ProfileIds::High;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MainGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::Mpeg2ProfileIds::Main;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SignalNoiseRatioScalableGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::Mpeg2ProfileIds::SignalNoiseRatioScalable;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SimpleGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::Mpeg2ProfileIds::Simple;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SpatiallyScalableGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;

      try 
      {
        int result = ::Windows::Media::MediaProperties::Mpeg2ProfileIds::SpatiallyScalable;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    

  private:
    ::Windows::Media::MediaProperties::Mpeg2ProfileIds^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMpeg2ProfileIds(::Windows::Media::MediaProperties::Mpeg2ProfileIds^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::Mpeg2ProfileIds^ UnwrapMpeg2ProfileIds(Handle<Value> value);
    friend bool IsMpeg2ProfileIdsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> Mpeg2ProfileIds::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMpeg2ProfileIds(::Windows::Media::MediaProperties::Mpeg2ProfileIds^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(Mpeg2ProfileIds::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::Mpeg2ProfileIds^ UnwrapMpeg2ProfileIds(Handle<Value> value)
  {
     return Mpeg2ProfileIds::Unwrap<Mpeg2ProfileIds>(value.As<Object>())->_instance;
  }

  void InitMpeg2ProfileIds(Handle<Object> exports)
  {
    Mpeg2ProfileIds::Init(exports);
  }

  class VideoEncodingProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("VideoEncodingProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("setFormatUserData"), FunctionTemplate::New(SetFormatUserData)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getFormatUserData"), FunctionTemplate::New(GetFormatUserData)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("width"), WidthGetter, WidthSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("height"), HeightGetter, HeightSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("bitrate"), BitrateGetter, BitrateSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("frameRate"), FrameRateGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("pixelAspectRatio"), PixelAspectRatioGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtype"), SubtypeGetter, SubtypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("properties"), PropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("type"), TypeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("profileId"), ProfileIdGetter, ProfileIdSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createH264"), FunctionTemplate::New(CreateH264)->GetFunction());
      constructor->Set(String::NewSymbol("createMpeg2"), FunctionTemplate::New(CreateMpeg2)->GetFunction());
      constructor->Set(String::NewSymbol("createUncompressed"), FunctionTemplate::New(CreateUncompressed)->GetFunction());

      exports->Set(String::NewSymbol("VideoEncodingProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    VideoEncodingProperties(::Windows::Media::MediaProperties::VideoEncodingProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::VideoEncodingProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::VideoEncodingProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaProperties::VideoEncodingProperties();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      VideoEncodingProperties *wrapperInstance = new VideoEncodingProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> SetFormatUserData(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(args.This());

      if (args.Length() == 1
        && (NodeRT::Utils::IsWinRtWrapperOf<::Platform::Array<unsigned char>^>(args[0]) || args[0]->IsArray()))
      {
        try
        {
          ::Platform::Array<unsigned char>^ arg0 = 
            [] (v8::Handle<v8::Value> value) -> ::Platform::Array<unsigned char>^
            {
              if (value->IsArray())
              {
                return NodeRT::Collections::JsArrayToWinrtArray<unsigned char>(value.As<Array>(), 
                 [](Handle<Value> value) -> bool {
                   return value->IsInt32();
                 },
                 [](Handle<Value> value) -> unsigned char {
                   return static_cast<unsigned char>(value->Int32Value());
                 }
                );
              }
              else
              {
                return dynamic_cast<::Platform::Array<unsigned char>^>(NodeRT::Utils::GetObjectInstance(value));
              }
            } (args[0]);
          
          wrapper->_instance->SetFormatUserData(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetFormatUserData(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Platform::Array<unsigned char>^ arg0;
          
          wrapper->_instance->GetFormatUserData(&arg0);
          Handle<Object> resObj = Object::New();
          resObj->Set(String::NewSymbol("value"), NodeRT::Collections::ArrayWrapper<unsigned char>::CreateArrayWrapper(arg0, 
            [](unsigned char val) -> Handle<Value> {
              return Integer::New(val);
            },
            [](Handle<Value> value) -> bool {
              return value->IsInt32();
            },
            [](Handle<Value> value) -> unsigned char {
              return static_cast<unsigned char>(value->Int32Value());
            }
          ));
          return scope.Close(resObj);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }


    static Handle<Value> CreateH264(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::MediaProperties::VideoEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::VideoEncodingProperties::CreateH264();
          return scope.Close(WrapVideoEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateMpeg2(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::MediaProperties::VideoEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::VideoEncodingProperties::CreateMpeg2();
          return scope.Close(WrapVideoEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateUncompressed(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 3
        && args[0]->IsString()
        && args[1]->IsUint32()
        && args[2]->IsUint32())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          unsigned int arg1 = static_cast<unsigned int>(args[1]->IntegerValue());
          unsigned int arg2 = static_cast<unsigned int>(args[2]->IntegerValue());
          
          ::Windows::Media::MediaProperties::VideoEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::VideoEncodingProperties::CreateUncompressed(arg0, arg1, arg2);
          return scope.Close(WrapVideoEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> WidthGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Width;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void WidthSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return;
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Width = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> HeightGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Height;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void HeightSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return;
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Height = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BitrateGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Bitrate;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void BitrateSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return;
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Bitrate = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> FrameRateGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaRatio^ result = wrapper->_instance->FrameRate;
        return scope.Close(WrapMediaRatio(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> PixelAspectRatioGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaRatio^ result = wrapper->_instance->PixelAspectRatio;
        return scope.Close(WrapMediaRatio(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SubtypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtype;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return;
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtype = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> PropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaPropertySet^ result = wrapper->_instance->Properties;
        return scope.Close(WrapMediaPropertySet(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Type;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ProfileIdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        int result = wrapper->_instance->ProfileId;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ProfileIdSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(info.This()))
      {
        return;
      }

      VideoEncodingProperties *wrapper = VideoEncodingProperties::Unwrap<VideoEncodingProperties>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->ProfileId = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::VideoEncodingProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapVideoEncodingProperties(::Windows::Media::MediaProperties::VideoEncodingProperties^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::VideoEncodingProperties^ UnwrapVideoEncodingProperties(Handle<Value> value);
    friend bool IsVideoEncodingPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> VideoEncodingProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapVideoEncodingProperties(::Windows::Media::MediaProperties::VideoEncodingProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(VideoEncodingProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::VideoEncodingProperties^ UnwrapVideoEncodingProperties(Handle<Value> value)
  {
     return VideoEncodingProperties::Unwrap<VideoEncodingProperties>(value.As<Object>())->_instance;
  }

  void InitVideoEncodingProperties(Handle<Object> exports)
  {
    VideoEncodingProperties::Init(exports);
  }

  class ImageEncodingProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageEncodingProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("width"), WidthGetter, WidthSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("height"), HeightGetter, HeightSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtype"), SubtypeGetter, SubtypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("properties"), PropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("type"), TypeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createUncompressed"), FunctionTemplate::New(CreateUncompressed)->GetFunction());
      constructor->Set(String::NewSymbol("createBmp"), FunctionTemplate::New(CreateBmp)->GetFunction());
      constructor->Set(String::NewSymbol("createJpeg"), FunctionTemplate::New(CreateJpeg)->GetFunction());
      constructor->Set(String::NewSymbol("createPng"), FunctionTemplate::New(CreatePng)->GetFunction());
      constructor->Set(String::NewSymbol("createJpegXR"), FunctionTemplate::New(CreateJpegXR)->GetFunction());

      exports->Set(String::NewSymbol("ImageEncodingProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageEncodingProperties(::Windows::Media::MediaProperties::ImageEncodingProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::ImageEncodingProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::ImageEncodingProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaProperties::ImageEncodingProperties();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageEncodingProperties *wrapperInstance = new ImageEncodingProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> CreateUncompressed(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::MediaPixelFormat arg0 = static_cast<::Windows::Media::MediaProperties::MediaPixelFormat>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::ImageEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::ImageEncodingProperties::CreateUncompressed(arg0);
          return scope.Close(WrapImageEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateBmp(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::MediaProperties::ImageEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::ImageEncodingProperties::CreateBmp();
          return scope.Close(WrapImageEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateJpeg(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::MediaProperties::ImageEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::ImageEncodingProperties::CreateJpeg();
          return scope.Close(WrapImageEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreatePng(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::MediaProperties::ImageEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::ImageEncodingProperties::CreatePng();
          return scope.Close(WrapImageEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateJpegXR(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Media::MediaProperties::ImageEncodingProperties^ result;
          result = ::Windows::Media::MediaProperties::ImageEncodingProperties::CreateJpegXR();
          return scope.Close(WrapImageEncodingProperties(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> WidthGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Width;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void WidthSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return;
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Width = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> HeightGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->Height;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void HeightSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return;
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->Height = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> SubtypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtype;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return;
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtype = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> PropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaPropertySet^ result = wrapper->_instance->Properties;
        return scope.Close(WrapMediaPropertySet(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ImageEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageEncodingProperties *wrapper = ImageEncodingProperties::Unwrap<ImageEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Type;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::ImageEncodingProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageEncodingProperties(::Windows::Media::MediaProperties::ImageEncodingProperties^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::ImageEncodingProperties^ UnwrapImageEncodingProperties(Handle<Value> value);
    friend bool IsImageEncodingPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageEncodingProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageEncodingProperties(::Windows::Media::MediaProperties::ImageEncodingProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageEncodingProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::ImageEncodingProperties^ UnwrapImageEncodingProperties(Handle<Value> value)
  {
     return ImageEncodingProperties::Unwrap<ImageEncodingProperties>(value.As<Object>())->_instance;
  }

  void InitImageEncodingProperties(Handle<Object> exports)
  {
    ImageEncodingProperties::Init(exports);
  }

  class ContainerEncodingProperties : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ContainerEncodingProperties"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("subtype"), SubtypeGetter, SubtypeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("properties"), PropertiesGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("type"), TypeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ContainerEncodingProperties"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ContainerEncodingProperties(::Windows::Media::MediaProperties::ContainerEncodingProperties^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::ContainerEncodingProperties^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::ContainerEncodingProperties^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaProperties::ContainerEncodingProperties();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ContainerEncodingProperties *wrapperInstance = new ContainerEncodingProperties(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> SubtypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ContainerEncodingProperties *wrapper = ContainerEncodingProperties::Unwrap<ContainerEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Subtype;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SubtypeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(info.This()))
      {
        return;
      }

      ContainerEncodingProperties *wrapper = ContainerEncodingProperties::Unwrap<ContainerEncodingProperties>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Subtype = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> PropertiesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ContainerEncodingProperties *wrapper = ContainerEncodingProperties::Unwrap<ContainerEncodingProperties>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::MediaPropertySet^ result = wrapper->_instance->Properties;
        return scope.Close(WrapMediaPropertySet(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> TypeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ContainerEncodingProperties *wrapper = ContainerEncodingProperties::Unwrap<ContainerEncodingProperties>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Type;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::ContainerEncodingProperties^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapContainerEncodingProperties(::Windows::Media::MediaProperties::ContainerEncodingProperties^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::ContainerEncodingProperties^ UnwrapContainerEncodingProperties(Handle<Value> value);
    friend bool IsContainerEncodingPropertiesWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ContainerEncodingProperties::s_constructorTemplate;

  v8::Handle<v8::Value> WrapContainerEncodingProperties(::Windows::Media::MediaProperties::ContainerEncodingProperties^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ContainerEncodingProperties::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::ContainerEncodingProperties^ UnwrapContainerEncodingProperties(Handle<Value> value)
  {
     return ContainerEncodingProperties::Unwrap<ContainerEncodingProperties>(value.As<Object>())->_instance;
  }

  void InitContainerEncodingProperties(Handle<Object> exports)
  {
    ContainerEncodingProperties::Init(exports);
  }

  class MediaEncodingProfile : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("MediaEncodingProfile"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("video"), VideoGetter, VideoSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("container"), ContainerGetter, ContainerSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("audio"), AudioGetter, AudioSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createWav"), FunctionTemplate::New(CreateWav)->GetFunction());
      constructor->Set(String::NewSymbol("createAvi"), FunctionTemplate::New(CreateAvi)->GetFunction());
      constructor->Set(String::NewSymbol("createM4a"), FunctionTemplate::New(CreateM4a)->GetFunction());
      constructor->Set(String::NewSymbol("createMp3"), FunctionTemplate::New(CreateMp3)->GetFunction());
      constructor->Set(String::NewSymbol("createWma"), FunctionTemplate::New(CreateWma)->GetFunction());
      constructor->Set(String::NewSymbol("createMp4"), FunctionTemplate::New(CreateMp4)->GetFunction());
      constructor->Set(String::NewSymbol("createWmv"), FunctionTemplate::New(CreateWmv)->GetFunction());
      func = FunctionTemplate::New(CreateFromFileAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      constructor->Set(String::NewSymbol("createFromFileAsync"), func);
      func = FunctionTemplate::New(CreateFromStreamAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      constructor->Set(String::NewSymbol("createFromStreamAsync"), func);

      exports->Set(String::NewSymbol("MediaEncodingProfile"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    MediaEncodingProfile(::Windows::Media::MediaProperties::MediaEncodingProfile^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Media::MediaProperties::MediaEncodingProfile^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Media::MediaProperties::MediaEncodingProfile^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 0)
      {
        try
        {
          winRtInstance = ref new ::Windows::Media::MediaProperties::MediaEncodingProfile();
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      MediaEncodingProfile *wrapperInstance = new MediaEncodingProfile(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  

    static Handle<Value> CreateFromFileAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ::Windows::Foundation::IAsyncOperation<::Windows::Media::MediaProperties::MediaEncodingProfile^>^ op;
      

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::IStorageFile^>(args[0]))
      {
        try
        {
          ::Windows::Storage::IStorageFile^ arg0 = dynamic_cast<::Windows::Storage::IStorageFile^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateFromFileAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Media::MediaProperties::MediaEncodingProfile^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {

            
            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = WrapMediaEncodingProfile(result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
          
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateFromStreamAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ::Windows::Foundation::IAsyncOperation<::Windows::Media::MediaProperties::MediaEncodingProfile^>^ op;
      

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::Streams::IRandomAccessStream^>(args[0]))
      {
        try
        {
          ::Windows::Storage::Streams::IRandomAccessStream^ arg0 = dynamic_cast<::Windows::Storage::Streams::IRandomAccessStream^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          op = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateFromStreamAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Media::MediaProperties::MediaEncodingProfile^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {

            
            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = WrapMediaEncodingProfile(result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
          
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }

    static Handle<Value> CreateWav(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::AudioEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::AudioEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateWav(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateAvi(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::VideoEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::VideoEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateAvi(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateM4a(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::AudioEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::AudioEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateM4a(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateMp3(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::AudioEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::AudioEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateMp3(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateWma(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::AudioEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::AudioEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateWma(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateMp4(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::VideoEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::VideoEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateMp4(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateWmv(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Media::MediaProperties::VideoEncodingQuality arg0 = static_cast<::Windows::Media::MediaProperties::VideoEncodingQuality>(args[0]->Int32Value());
          
          ::Windows::Media::MediaProperties::MediaEncodingProfile^ result;
          result = ::Windows::Media::MediaProperties::MediaEncodingProfile::CreateWmv(arg0);
          return scope.Close(WrapMediaEncodingProfile(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> VideoGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MediaEncodingProfile *wrapper = MediaEncodingProfile::Unwrap<MediaEncodingProfile>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::VideoEncodingProperties^ result = wrapper->_instance->Video;
        return scope.Close(WrapVideoEncodingProperties(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void VideoSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::VideoEncodingProperties^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(info.This()))
      {
        return;
      }

      MediaEncodingProfile *wrapper = MediaEncodingProfile::Unwrap<MediaEncodingProfile>(info.This());

      try 
      {
        
        ::Windows::Media::MediaProperties::VideoEncodingProperties^ winRtValue = dynamic_cast<::Windows::Media::MediaProperties::VideoEncodingProperties^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->Video = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContainerGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MediaEncodingProfile *wrapper = MediaEncodingProfile::Unwrap<MediaEncodingProfile>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::ContainerEncodingProperties^ result = wrapper->_instance->Container;
        return scope.Close(WrapContainerEncodingProperties(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ContainerSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(info.This()))
      {
        return;
      }

      MediaEncodingProfile *wrapper = MediaEncodingProfile::Unwrap<MediaEncodingProfile>(info.This());

      try 
      {
        
        ::Windows::Media::MediaProperties::ContainerEncodingProperties^ winRtValue = dynamic_cast<::Windows::Media::MediaProperties::ContainerEncodingProperties^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->Container = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AudioGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      MediaEncodingProfile *wrapper = MediaEncodingProfile::Unwrap<MediaEncodingProfile>(info.This());

      try 
      {
        ::Windows::Media::MediaProperties::AudioEncodingProperties^ result = wrapper->_instance->Audio;
        return scope.Close(WrapAudioEncodingProperties(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AudioSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::AudioEncodingProperties^>(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Media::MediaProperties::MediaEncodingProfile^>(info.This()))
      {
        return;
      }

      MediaEncodingProfile *wrapper = MediaEncodingProfile::Unwrap<MediaEncodingProfile>(info.This());

      try 
      {
        
        ::Windows::Media::MediaProperties::AudioEncodingProperties^ winRtValue = dynamic_cast<::Windows::Media::MediaProperties::AudioEncodingProperties^>(NodeRT::Utils::GetObjectInstance(value));

        wrapper->_instance->Audio = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Media::MediaProperties::MediaEncodingProfile^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapMediaEncodingProfile(::Windows::Media::MediaProperties::MediaEncodingProfile^ wintRtInstance);
    friend ::Windows::Media::MediaProperties::MediaEncodingProfile^ UnwrapMediaEncodingProfile(Handle<Value> value);
    friend bool IsMediaEncodingProfileWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> MediaEncodingProfile::s_constructorTemplate;

  v8::Handle<v8::Value> WrapMediaEncodingProfile(::Windows::Media::MediaProperties::MediaEncodingProfile^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(MediaEncodingProfile::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Media::MediaProperties::MediaEncodingProfile^ UnwrapMediaEncodingProfile(Handle<Value> value)
  {
     return MediaEncodingProfile::Unwrap<MediaEncodingProfile>(value.As<Object>())->_instance;
  }

  void InitMediaEncodingProfile(Handle<Object> exports)
  {
    MediaEncodingProfile::Init(exports);
  }

} } } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::Media::MediaProperties::InitMediaPixelFormatEnum(exports);
  NodeRT::Windows::Media::MediaProperties::InitMediaThumbnailFormatEnum(exports);
  NodeRT::Windows::Media::MediaProperties::InitAudioEncodingQualityEnum(exports);
  NodeRT::Windows::Media::MediaProperties::InitVideoEncodingQualityEnum(exports);
  NodeRT::Windows::Media::MediaProperties::InitMediaRatio(exports);
  NodeRT::Windows::Media::MediaProperties::InitMediaPropertySet(exports);
  NodeRT::Windows::Media::MediaProperties::InitIMediaEncodingProperties(exports);
  NodeRT::Windows::Media::MediaProperties::InitAudioEncodingProperties(exports);
  NodeRT::Windows::Media::MediaProperties::InitMediaEncodingSubtypes(exports);
  NodeRT::Windows::Media::MediaProperties::InitH264ProfileIds(exports);
  NodeRT::Windows::Media::MediaProperties::InitMpeg2ProfileIds(exports);
  NodeRT::Windows::Media::MediaProperties::InitVideoEncodingProperties(exports);
  NodeRT::Windows::Media::MediaProperties::InitImageEncodingProperties(exports);
  NodeRT::Windows::Media::MediaProperties::InitContainerEncodingProperties(exports);
  NodeRT::Windows::Media::MediaProperties::InitMediaEncodingProfile(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.Media.MediaProperties", exports);
}


NODE_MODULE(binding, init)