declare module "windows.media.mediaproperties" {
  export enum MediaPixelFormat {
    nv12,
    bgra8,
  }

  export enum MediaThumbnailFormat {
    bmp,
    bgra8,
  }

  export enum AudioEncodingQuality {
    auto,
    high,
    medium,
    low,
  }

  export enum VideoEncodingQuality {
    auto,
    hD1080p,
    hD720p,
    wvga,
    ntsc,
    pal,
    vga,
    qvga,
  }

  export class MediaRatio {
    numerator: Number;
    denominator: Number;
    constructor();

  }

  export class MediaPropertySet {
    constructor();

    lookup(key: String): Object;

    hasKey(key: String): Boolean;

    getView(): Object;

    insert(key: String, value: Object): Boolean;

    remove(key: String): void;

    clear(): void;

    first(): Object;

  }

  export class IMediaEncodingProperties {
    properties: MediaPropertySet;
    subtype: String;
    type: String;
    constructor();

  }

  export class AudioEncodingProperties {
    subtype: String;
    properties: MediaPropertySet;
    type: String;
    sampleRate: Number;
    channelCount: Number;
    bitsPerSample: Number;
    bitrate: Number;
    constructor();

    static createAac(sampleRate: Number, channelCount: Number, bitrate: Number): AudioEncodingProperties;


    static createAacAdts(sampleRate: Number, channelCount: Number, bitrate: Number): AudioEncodingProperties;


    static createMp3(sampleRate: Number, channelCount: Number, bitrate: Number): AudioEncodingProperties;


    static createPcm(sampleRate: Number, channelCount: Number, bitsPerSample: Number): AudioEncodingProperties;


    static createWma(sampleRate: Number, channelCount: Number, bitrate: Number): AudioEncodingProperties;


    setFormatUserData(value: Array<Number>): void;

    getFormatUserData(value: Array<Number>): void;

  }

  export class MediaEncodingSubtypes {
    static aac: String;
    static aacAdts: String;
    static ac3: String;
    static amrNb: String;
    static amrWb: String;
    static argb32: String;
    static asf: String;
    static avi: String;
    static bgra8: String;
    static bmp: String;
    static float: String;
    static gif: String;
    static h263: String;
    static h264: String;
    static h264Es: String;
    static hevc: String;
    static hevcEs: String;
    static iyuv: String;
    static jpeg: String;
    static jpegXr: String;
    static mjpg: String;
    static mp3: String;
    static mpeg: String;
    static mpeg1: String;
    static mpeg2: String;
    static mpeg4: String;
    static nv12: String;
    static pcm: String;
    static png: String;
    static rgb32: String;
    static tiff: String;
    static wave: String;
    static wma8: String;
    static wma9: String;
    static wmv3: String;
    static wvc1: String;
    static yuy2: String;
    static yv12: String;
    static rgb24: String;
    static eac3: String;
    constructor();

  }

  export class H264ProfileIds {
    static baseline: Number;
    static constrainedBaseline: Number;
    static extended: Number;
    static high: Number;
    static high10: Number;
    static high422: Number;
    static high444: Number;
    static main: Number;
    static multiviewHigh: Number;
    static stereoHigh: Number;
    constructor();

  }

  export class Mpeg2ProfileIds {
    static high: Number;
    static main: Number;
    static signalNoiseRatioScalable: Number;
    static simple: Number;
    static spatiallyScalable: Number;
    constructor();

  }

  export class VideoEncodingProperties {
    width: Number;
    height: Number;
    bitrate: Number;
    frameRate: MediaRatio;
    pixelAspectRatio: MediaRatio;
    subtype: String;
    properties: MediaPropertySet;
    type: String;
    profileId: Number;
    constructor();

    static createH264(): VideoEncodingProperties;


    static createMpeg2(): VideoEncodingProperties;


    static createUncompressed(subtype: String, width: Number, height: Number): VideoEncodingProperties;


    setFormatUserData(value: Array<Number>): void;

    getFormatUserData(value: Array<Number>): void;

  }

  export class ImageEncodingProperties {
    width: Number;
    height: Number;
    subtype: String;
    properties: MediaPropertySet;
    type: String;
    constructor();

    static createUncompressed(format: MediaPixelFormat): ImageEncodingProperties;


    static createBmp(): ImageEncodingProperties;


    static createJpeg(): ImageEncodingProperties;


    static createPng(): ImageEncodingProperties;


    static createJpegXR(): ImageEncodingProperties;


  }

  export class ContainerEncodingProperties {
    subtype: String;
    properties: MediaPropertySet;
    type: String;
    constructor();

  }

  export class MediaEncodingProfile {
    video: VideoEncodingProperties;
    container: ContainerEncodingProperties;
    audio: AudioEncodingProperties;
    constructor();

    static createFromFileAsync(file: Object, callback: (error: Error, result: MediaEncodingProfile) => void): void ;


    static createFromStreamAsync(stream: Object, callback: (error: Error, result: MediaEncodingProfile) => void): void ;


    static createWav(quality: AudioEncodingQuality): MediaEncodingProfile;


    static createAvi(quality: VideoEncodingQuality): MediaEncodingProfile;


    static createM4a(quality: AudioEncodingQuality): MediaEncodingProfile;


    static createMp3(quality: AudioEncodingQuality): MediaEncodingProfile;


    static createWma(quality: AudioEncodingQuality): MediaEncodingProfile;


    static createMp4(quality: VideoEncodingQuality): MediaEncodingProfile;


    static createWmv(quality: VideoEncodingQuality): MediaEncodingProfile;


  }

}



